/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/*
 *
 * Copyright (c) 1996 Brian Maso. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted. 
 * 
 * An AutoSignal is a type of Signal. It provides methods to make the Signal
 * signaled or unsignaled externally. The inherited Signal method
 * implementations one properly calls setState() to modify the state.
 * The difference with Signal is that an
 * AutoSignal immediately becomes unsignaled when a single Thread
 * that has been waiting for it is released. (Ie, when the AutoSignal is
 * gained, it automatically becomes unsignaled).
 *
 * An AutoSignal is much like a Mutex, except that the AutoSignal is never
 * automatically signaled. Signalling must happen explicitly. To illustrate,
 * This code section uses a Mutex, and will run continuously without
 * waiting once the Mutex is owned by the running Thread:
 *
 * while(true) {
 *      Scheduler.waitFor(myMutex);
 *      // do something here.
 * }
 *
 * On the other hand, this loop will potentially wait during every iteration:
 *
 * while(true) {
 *      Scheduler.waitFor(myAutoSignal);
 *      // do something here.
 * }
 *
 */

package com.radview.util.synch;

/**
 * An AutoSignal is a minor variation on the Signal class. The only
 * difference is that an AutoSignal switches to the unsignaled state
 * every time a thread waits for it in the Scheduler and is released. That is,
 * the AutoSignal's state must be set to signaled explicitly using
 * <I>Signal.signal()</I> for each thread that is waiting for the AutoSignal
 * to be released.
 *
 * <P> There are no public methods implemented in the AutoSignal class other than
 * the constructors, which mirror the constructors of the Signal class.
 * Use the Signal class methods to modify the state of an AutoSignal object.
 *
 * @see Signal
 *
 * @version 1.0
 * @author Brian Maso
 */

public class AutoSignal extends Signal {

	/**
	 * The AutoSignal is created in the unsignaled state
	 * by default.
	 */
	public AutoSignal() {
		super();
		return;
	}
	
	/**
	 * Pass in the initial state of the AutoSignal. Possible initial
	 * states are <I>Synch.signaled</I> and <I>Synch.unsignaled</I>.
	 *
	 * @param	state		<UL>The initial state has one of two possible
	 *				values:
	 *				<LI> <I>Synch.signaled</I> - initial state is
	 *				unsignaled. This is the default.
	 *				<LI> <I>Synch.signaled</I> - initial state is
	 *				signaled.
	 */
	public AutoSignal(int state) {
		super(state);
		return;
	}
	
      // When the AutoSignal is gained (ie, when a thread that is waiting
      // for the AutoSignal is released by the Scheduler), the state
      // of the AutoSignal is automatically changed to unsignaled.
	void gain(Thread t) {
		super.gain(t);
		setState(unsignaled);
		return;
	}
}
