/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
 
/*
 *
 * Copyright (c) 1996 Brian Maso. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted. 
 *
 * A CounterSynch simply maintains an internal count. Each gaining
 * of the CounterSynch decrements the number. When the number is
 * 0, the CounterSynch is unsignaled. A public method, increment(),
 * allows you to increase the count. This is a good object to
 * control synchronized access to a cubby-hole or list:
 *
 * class CubbyHole {
 *     private Vector _contents = new Vector();
 *     CounterSynch _csynch = new CounterSynch(0);
 *
 *     public Object get() {
 *         Scheduler.waitFor(_csynch);
 *         Object ret = _contents.getElementAt(0);
 *         _contents.removeElementAt(0);
 *         return ret;
 *     }
 *
 *     public void put(Object obj) {
 *         _contents.addElement(obj);
 *         _csynch.increment();
 *         return;
 *     }
 * }
 *
 */

package com.radview.util.synch;


/**
 * A CounterSynch maintains an internal usage count. The CounterSynch
 * is signaled as long as the internal usage count is above zero. The
 * usage count is decremented every time a thread that is waiting for the
 * CounterSynch in the Scheduler (using <I>Scheduler.waitFor()</I>) is
 * released. The usage count is incremented by an explicit call to
 * <I>CounterSynch.increment()</I>.
 *
 * One possible use for a CounterSynch is to manage access to a synchroized
 * storage device. Here is some simple code for a SynchedStack, a LIFO device
 * which suspends any thread requesting a pop off the stack until the stack
 * is not empty:
 *
 * <PRE>
 *
 *	public class SynchedStack {
 *		private Vector _vectStack = new Vector();
 *		private CounterSynch _cs = new CounterSynch(0);
 *
 *		public SynchedStack() {}
 *
 *		public synchronized push(Object o) {
 *			_vectStack.insertElementAt(0);
 *			_cs.increment();
 *		}
 *
 *		// Note that pop() is not a
 *		// synchronized method. That's because
 *		// a thread suspended in the Scheduler would
 *		// still own the monitor for the SynchedStack,
 *		// so no other thread would ever be able
 *		// to run - a deadlock condition. 
 *
 *		public Object pop() {
 *			int ret;
 *			ret = Scheduler.waitFor(_cs);
 *
 *			synchronized(this) {
 *				if(Scheduler.ABANDONED == ret)
 *					return null;
 *
 *				Object o = _vectStack.elementAt(0);
 *				_vectStack.removeElementAt(0);
 *				return o;
 *			}
 *		}
 *	}
 *
 * @version 1.0
 * @author Brian Maso
 */

public class CounterSynch extends Synch {
	int _count;

	/**
	 * Constructor for a CounterSynch with an initial usage
	 * count of 0. The CounterSynch will be unsignaled
	 * until an explicit call to <I>increment()</I> is made.
	 */
	public CounterSynch() {
		this(0);
	}

	/**
	 * CounterSynch constructor which allows you to specify the
	 * initial usage count. If <I>cInitCount</I> is greater than
	 * zero, then the CounterSynch's state will be signaled initially.
	 *
	 * @param	cInitCount		Initial usage count of the CounterSynch
	 *					object.
	 */
	public CounterSynch(int cInitCount) {
		super(0 <= cInitCount ? unsignaled : signaled);
		_count = cInitCount;
		return;
	}
	
	synchronized boolean reserve(Thread t) {
		if(!super.reserve(t))
			return false;
			
		if(1 == _count)
			setState(reserved);
			
		return true;
	}
	
	synchronized void unreserve(Thread t) {
		if(1 == _count)
			setState(signaled);
		return;
	}
	
	synchronized void gain(Thread t) {
		if(0 == --_count)
			setState(unsignaled);
		return;
	}
	
	/**
	 * Increments the CounterSynch's internal usage
	 * count by 1. If the CounterSynch was unsignaled,
	 * it will be signaled after a call to this method.
	 */
	public synchronized void increment() {
		_count++;
		setState(signaled);
	}
}
	
