/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

 
/*
 *
 * Copyright (c) 1996 Brian Maso. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and
 * without fee is hereby granted. 
 * 
 * A Timer is a type of simple Synch. It is initiallized with a
 * timeout, in milliseconds. The Timer is initially unsignaled.
 * When time runs out, the Timer becomes signaled, and all Threads
 * waiting for it are resumed. Alternatively, the Timer may be
 * initially signaled and become unsignaled after the time period
 * has elapsed.
 *
 * The Timer works by creating a new Thread which simply waits
 * for the appropriate amount of time. If the Timer is destroyed
 * before time has run out, the Timer's thread reacts appropriately
 * be destroying itself.
 *
 */

package com.radview.util.synch;

/**
 * A Timer is a Synch which switches state after a specified interval of
 * time has elapsed. The Timer can be configured to be initially unsignaled
 * and become signaled after the time interval, or to be initially
 * signaled and become unsignaled after the time interval.
 *
 * @version 1.0
 * @author Brian Maso
 */

public class Timer extends Synch implements Runnable {
	long _ms;
	Thread _thread;
	boolean _fCountUp;
	
	/**
	 * Constructor for a "count-up" Timer. That is, a Timer which is initially
	 * unsignaled and becomes signaled after <I>ms</I> milliseconds has
	 * elapsed. Note that the Timer is only guaranteed to switch states at
	 * some time after the <I>ms</I> millsecond interval. So don't try to
	 * use a Timer for real-time applications.
	 *
	 * @param	ms		Number of millseconds before the Timer becomes
	 *				signaled. Passing a negative number could cause
	 *				the Timer to wait for 40 days and 40 nights (or
	 *				longer)!
	 */
	public Timer(long ms) {
		this(ms, true);
	}

	/**
	 * Constructor for either a "count-up" or "count-down" Timer. A count-up
	 * Timer is initially unsignaled and becomes signaled after <I>ms</I>
	 * milliseconds. A count-down Timer is initially signaled and becomes
	 * unsignaled after the <I>ms</I> millsecond duration.
	 *
	 * @param	ms		Number of millseconds before the Timer becomes
	 *				signaled. Passing a negative number could cause
	 *				the Timer to wait for 40 days and 40 nights (or
	 *				longer)!
	 * @param	fCountUp	If true, then the Timer is a count-up Timer. If false,
	 *				the Timer is a count-down Timer.
	 */
	public Timer(long ms, boolean fCountUp) {
		super(fCountUp ? unsignaled : signaled);
		_fCountUp = fCountUp;
		_ms = ms;
		_thread = new Thread(this);
		_thread.setDaemon(true);
		_thread.start();
	}
	
	/* *********
	 * void gain(Thread)
	 *
	 * The Synch class implementation of this method
	 * automatically stores OWNERSHIP under for this
	 * Thread. Timers, however, have no concept of ownership,
	 * and so should get rid of this flag.
	 *
	 ********* */
	
	void gain(Thread t) {
		super.gain(t);
		_hashOwnership.remove(t);
	}
	
	/* *********
	 * void run()
	 *
	 * The Thread created by this object runs within this method.
	 * The job of this method is to simply wait _ms milliseconds,
	 * and make this Synch signaled if that time has elasped
	 * without the Timer having been destroyed. This Thread
	 * knows the Timer has been destroyed because the _thread
	 * value of this object will be null.
	 *
	 ********* */
	
	public void run() {
		// Make sure Timer hasn't already been destroyed.
		if(null == _thread)
			return;
			
		// wait for _ms milliseconds.
		try {
			Thread.currentThread().sleep(_ms);
		} catch(Exception e) {}
		
		// If _thread is null, then Timer was destroyed
		// while we were sleeping.
		if(null == _thread)
			return;
			
		// Switch Synch state before returning.
		setState(_fCountUp ? signaled : unsignaled);
		return;
	}
	
	/* *********
	 * void destroy()
	 *
	 * Overrider of the Synch.destroy() method. After calling
	 * Synch implementation, the _thread variable is set to null.
	 * When the running thread is awakened, the null value lets
	 * the Thread know this Timer has been destroyed.
	 *
	 ********* */
	
	/**
	 * Destroys the Timer. Note that the Timer's background
	 * thread may persist for some time, but it will not interfere
	 * with the execution or orderly shutdown of your application.
	 */
	public synchronized void destroy() {
		super.destroy();
		_thread = null;
	}
}
