/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __CONNPOOL_H
#define __CONNPOOL_H

#include <string.h> 
#include <sys/types.h>
#include <sys/timeb.h>
#include <errno.h>
#include <RVCString.h>
#include <collstr.h>
#include <ordered.h>
#include <thr.h>
#include <thr.h> 

#ifdef __TV_UNIX__

#ifndef __TV_linux__
#ifndef INADDR_NONE
#define INADDR_NONE     0xffffffff
#endif /* INADDR_NONE */
#endif /* __TV_linux__ */

#if defined __TV_aix__ || defined __TV_linux__
#include <sys/ioctl.h>
#else
#include <sys/filio.h>
#endif

#include <sys/time.h>
#include <sys/socket.h>
#include <sys/utsname.h>
#include <unistd.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <stropts.h>
#include <poll.h>
#endif

#ifdef __TV_WINDOWS__
#include <winsock.h>
#endif

#include "httphost.h"
#include "gethostip.h"
#include "tvpsync.h"

class CWLCommandManager;
class CWLConnectionsPool;
class CWLSslConnection;
class CWLSSLContext;
class CWLWorkspace;
class CWLSlowConnection;
class CWLSSLCiphers;
class CWLTransactionsObject;
class CWLTransactionData;
class CWLHttpRequest;
class CWLGenericConnection;
class CWLSslCiphersCollection;

#define HOSTENT_MAX  2048

#define MAX_CONNECTIONS 8

#define BLOCKING			0
#define NON_BLOCKING	1

#define KEPT_ALIVE	2

#ifdef __TV_WINDOWS__
#define EINPROGRESS  WSAEINPROGRESS
#define EWOULDBLOCK  WSAEWOULDBLOCK
#define EALREADY     WSAEALREADY
#define ETIMEDOUT    WSAETIMEDOUT
#define EISCONN      WSAEISCONN
#define ECONNREFUSED WSAECONNREFUSED
#define ENOBUFS      WSAENOBUFS
#ifndef EINVAL
#define EINVAL       WSAEINVAL
#endif // EINVAL
#ifndef EADDRINUSE
#define EADDRINUSE WSAEADDRINUSE
#endif // EADDRINUSE
#ifndef EINTR
#define EINTR WSAEINTR
#endif // EINTR

#else	// (UNIX)
#define WSAEINPROGRESS  EINPROGRESS
#define WSAEWOULDBLOCK  EWOULDBLOCK
#define WSAEALREADY     EALREADY
#define WSAETIMEDOUT    ETIMEDOUT
#define WSAEISCONN      EISCONN
#define WSAECONNREFUSED ECONNREFUSED
#define WSAENOBUFS      ENOBUFS
#define WSAESHUTDOWN    ESHUTDOWN
#define WSAECONNRESET   ECONNRESET
#define WSAEADDRINUSE EADDRINUSE
#define WSAEINTR EINTR
#define WSAGetLastError() errno

#endif // __TV_WINDOWS__


//typedef struct HTTPLOCALADDR  {
//  struct in_addr* addr[64];
//  int             num;
//} HTTPLocalAddr;
class HTTPLocalAddr {
public:
  struct in_addr* addr[128];
  int             num;

	HTTPLocalAddr() {}
	~HTTPLocalAddr() { for (int i = 0; i < num; i++) free(addr[i]); }
};


extern void UnescapeString( const char* buffer,
							int &length,
							bool lowLevelUnescape,
							const char *userEscapeChar,
							RVCString &retbuf );


class CWLConnectionsPool {
private:
	RVOrdered           m_Pool;	// array of connections
	RVOrdered           m_SslContextPool;	// array of SSL contexts
	unsigned            m_MaxConnections;
	RVCriticalSection   m_ConnectionLock;

	// slow-connection variables
	CWLSlowConnection*  m_SlowConnection;

	// cipher suites variables
	CWLSslCiphersCollection* m_SslCipherSuites;

	CWLCommandManager*  m_Cm;

	RVCString           m_ErrorCode;

	// Object that contains all the host addresses.
	CWLDnsTable         m_DnsAddrTable;


public:
	in_addr             myin;

public:
// struct that contains all the local addresses
// static HTTPLocalAddr localaddr;
// instead of the above class, static class which contain all addresses, and support services.
  static GetHostIP    m_gLocalAddr;

	static	long GetMultiIPAddressIndex();
	static  long GetNextMultiIPAddressIndex();
private:
	// Chaim - MultiIP
	static RVMutexLock	m_IPIndexLock;
	static long			m_IPIndex;

public:
	RVCriticalSection   m_ConnectLock;	// for openConnection
	RVReadersWriterLock m_SslLock;
	RVHashDictionary    m_SslCacheDB;
	// Manage alternative semi ip for cases in which we can't translate host name to ip
	// (could happen when we are working with ssl through proxy) - Shahar 17-Dec-2000
	RVOrdered						m_SslAlternativeIP;

  CWLWorkspace*       m_Ws;

public:
	CWLConnectionsPool();
	~CWLConnectionsPool();

	void              Init(CWLCommandManager* cm, unsigned max);
	void              InitCiphers(unsigned strength);
	static int        GetLocalHosts();
	bool              GetSockAddr(sockaddr_in& sockaddr, RVCString& host, int port, bool resolveOnly, bool reportError);
	CWLGenericConnection* AcquireConnection( int protocol, const char* host, int port, int, CWLHttpRequest* pHttpReq );
	CWLSSLContext*    GetSslContext(int* sslErr, int* errCode);
	CWLSslCiphersCollection* GetCipherSuite() { return m_SslCipherSuites; }
	void ReleaseConnection(CWLGenericConnection*, int);
	void              CloseAllConnections();
	void              DeleteAllConnections() { m_Pool.clearAndDestroy(); }
	void              AtEndOfRound();
	void              ClearDnsCache();
	void              ClearSslCache() { m_SslCacheDB.clearAndDestroy(); }

	void              IncUsers();
	void              DecUsers();

        // Chaim - The override flag is for SSL. In SSL we want to convert the buffer
        // buffer not right before the send, but higher up, before calling SSLWrite
	int               Send(int sock, const char* buffer, int &len, int flags, int& lastRes, bool OverrideDecode = false);
	int               Recv(int sock, char* buffer, int len, int flags, int& lastRes);

	// slow-connection methods
	bool              UseSlowConnection(unsigned rate, unsigned threads);
	void              AddReport();

#ifdef __SSL__
	RVOrdered*        GetSSLContextPool() { return &m_SslContextPool; }
#endif

//#ifdef __OPEN_SSL__
	RVOrdered* GetConnectionsPool() { return &m_Pool; }
//#endif // __OPEN_SSL__

	int                ReportError(const char* frmName, int frmId, ...);
	void               SetErrorCode(int err);
  CWLCommandManager* GetCm()  {return m_Cm;}
private:
	CWLGenericConnection* createConnection( int protocol, sockaddr_in& addr, const char* host, int port, int, CWLHttpRequest* pHttpReq ); //returns the CWLSecureSslConnection or CWLInsecureConnection object deppending on protocol value

};



// this class simulates slow connections (modems) by calculating the
// time it should take to send/receive data on the slow connection.
#define DEF_RATE 14400
#define DEF_TX_RATE 14400
#define DEF_RX_RATE 14400
#define DEF_FACTOR 8
#define DEF_TX_FACTOR 8
#define DEF_RX_FACTOR 8
#define MAX_VECTOR_SIZE 25

class CWLSlowConnection {
private:
	bool                m_MTMode;	// multi-threaded mode
	unsigned            m_Rate;	// bps
	unsigned            m_SendRate;	// bps after compression
	unsigned            m_RecvRate;	// bps after compression
	double              m_SendBps;	// Bytes per second
	double              m_RecvBps;	// Bytes per second
	unsigned            m_Size;	// buffer size to send/receive
	unsigned            m_MinTime;	// time resolution (system dependent)
	unsigned            m_Users;	// how many connections are using the 'modem' concurrently
	bool                m_TimerSet;

	struct {
		double startTime;	// command's start time
		double endTime;		// command's end time
		double totalTime;	// total active time
		double totalBytes;
		double totalBytesMark;
	}                   m_Calc;

	struct {
		double startTime;		// start of command
		double endTime;			// end of command
		double totalTime;		// total active time
		double prevTime;		// previous total active time
		double totalBytes;	// reported bytes
		double prevBytes;
	}                   m_Report;

	double              m_TotalBytes;
	double              m_TotalTime;
	struct {
		double time;
		double bytes;
	}                   m_VecReport[MAX_VECTOR_SIZE];
	unsigned            m_VecIndex;

	RVCriticalSection   m_SocketLock;
	RVCriticalSection   m_UsersLock;
	RVCriticalSection   m_TimeLock;

	CWLConnectionsPool* m_Cp;
	CWLCommandManager*  m_Cm;

public:
	CWLSlowConnection();
	virtual ~CWLSlowConnection();

	bool      Init(CWLConnectionsPool* cp, CWLCommandManager* cm, unsigned rate, bool m_MTMode);

public:
	void      AtEndOfRound();
	int       Send(int sock, const char* buffer, int &len, int flags, int& lastRes, bool OverrideDecode = false);
	int       Recv(int sock, char* buffer, int len, int flags, int& lastRes);
	void      IncUsers();
	void      DecUsers();

	void      AddReport();

	unsigned  Speed();

private:
	unsigned  calcBytes();
	double    addBytes(int bytes);
	double    getBytes();
	void      setActiveTimer();
	void      resetActiveTimer();
	void      addReport(double currTime);
};

#endif // __CONNPOOL_H

