/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
package com.radview.loadengine.appletloader.system;

import java.io.*;
import java.util.HashMap;
import java.net.*;
import java.util.zip.*;

// Radview classes
import com.radview.util.log.Log;

/**
 * JarResources: JarResources maps all resources included in a
 * Zip or Jar file. Additionaly, it provides a method to extract one
 * as a blob.
 */
public class JarResources {

	// jar resource mapping tables
	private HashMap htJarContents=new HashMap();

	//
	// Make the JarResources one per system.
  private static JarResources systemJarResources = null;

	// #BEGIN: class static initializing block
	static {
		try {
			if (Log.isLogging()) Log.logCategory.debug("JarResources initializer");
			systemJarResources = new JarResources();
		}
		catch(Exception e)
		{
			if (Log.isLogging()) Log.logCategory.error("JarResources initializer error: " + e.toString());
		}
	}
	// a jar file
	//private String jarFileName;

	/**
	* creates a JarResources. It extracts all resources from a Jar
	* into an internal hashmap, keyed by resource names.
	* @param jarFileName a jar or zip file
	*/
	public JarResources() {
	}

	public static JarResources getSystemJarResources()
	{
		return systemJarResources;
	}

	/**
	* Extracts a jar resource as a blob.
	* @param name a resource name.
	*/
	public void setStream(String className, String codeBase, byte[] stream)
		throws MalformedURLException
	{
		URL codeBaseURL = new URL(codeBase);
		if (Log.isLogging()) Log.logCategory.debug("setStream(" + className+","+ codeBase+","+stream.length+")");

		if (className.endsWith(".jar")|| className.endsWith(".zip"))
		{
			if (Log.isLogging()) Log.logCategory.debug("calling setResources(" + className + ")");
			setResources(codeBase, stream);
		}
		else if (className.endsWith(".class"))
		{
//			String name = className.replace('/', '.');
			addClassResource(className, codeBaseURL, stream);
		}
		else
		{
			addClassResource(className, codeBaseURL, stream);
		}
	}

	public byte[] getResource(String name, URL codeBaseURL)
	{
		URL resourceURL;
		try {
			resourceURL = new URL(codeBaseURL, name);
		} catch (MalformedURLException e) {
			e.printStackTrace();
			return null;
		}
		if (Log.isLogging()) Log.logCategory.debug("getResource("+resourceURL.toString()+")");
		return (byte[])htJarContents.get(resourceURL.toString());
	}

	public boolean isResourceExists(String name, URL codeBaseURL)
		throws MalformedURLException
	{
		URL resourceURL = new URL(codeBaseURL, name);
		return htJarContents.containsKey(resourceURL.toString());
	}

	public byte[] getResource(String name) {
		return (byte[])htJarContents.get(name);
	}

	public boolean isResourceFullnameExists(String name) {
		return htJarContents.containsKey(name);
	}

	public void addClassResource(String resourceName, URL codeBaseURL, byte [] stream)
	{
		URL resourceURL;
		try {
			resourceURL = new URL(codeBaseURL, resourceName);
		} catch (MalformedURLException e) {
			e.printStackTrace();
			return;
		}
		String resName = resourceURL.toString();
		if (isResourceFullnameExists(resName)) {
			if (Log.isLogging()) Log.logCategory.debug("addClassResource("+resName+") already exists");
		} else {
			if (Log.isLogging()) Log.logCategory.debug("addClassResource("+resName+","+stream.length+")");
			//String name = appletName.replace('/', '.');
			synchronized (this) {
				if (htJarContents.get(resName) == null) {
			    htJarContents.put(resName, stream);
		}
  }
	}
	}

	/**
	* initializes internal hash tables with Jar file resources.
	*/
	public void setResources(String codeBase, byte [] jarStream) {
		try {
			if (Log.isLogging()) Log.logCategory.debug("resource len " + jarStream.length);
			URL codeBaseURL = new URL(codeBase);
			// extract resources and put them into the hashmap.
			//FileInputStream fis=new FileInputStream(jarFileName);
			ByteArrayInputStream bais = new ByteArrayInputStream(jarStream);
			BufferedInputStream bis=new BufferedInputStream(bais);
			ZipInputStream zis=new ZipInputStream(bis);
			ZipEntry ze=null;
			while ((ze=zis.getNextEntry())!=null) {
		 		if (ze.isDirectory()) {
					continue;
		 		}

				ByteArrayOutputStream bytearrayoutputstream = new ByteArrayOutputStream();
				BufferedInputStream bufferedinputstream = new BufferedInputStream(zis);

				for(int i = bufferedinputstream.read(); i != -1; i = bufferedinputstream.read())
				{
					bytearrayoutputstream.write(i);
				}
				//System.out.println(ze.getName()+" Done ");
				byte b[] = bytearrayoutputstream.toByteArray();

				String name = ze.getName();

				// add to internal resource hashmap
				addClassResource(name, codeBaseURL, b);
			}
	    } catch (NullPointerException e) {
				System.out.println("done.");
	    } catch (FileNotFoundException e) {
				e.printStackTrace();
	    } catch (IOException e) {
				e.printStackTrace();
	    } catch (Exception e) {
				e.printStackTrace();
	    }
	}

	/**
	* Dumps a zip entry into a string.
	* @param ze a ZipEntry
	*/
	private String dumpZipEntry(ZipEntry ze) {
		StringBuffer sb=new StringBuffer();
		if (ze.isDirectory()) {
			sb.append("d ");
		} else {
			sb.append("f ");
		}
		if (ze.getMethod()==ZipEntry.STORED) {
			sb.append("stored   ");
		} else {
			sb.append("defalted ");
		}
		sb.append(ze.getName());
		sb.append("\t");
		sb.append(""+ze.getSize());
		if (ze.getMethod()==ZipEntry.DEFLATED) {
			sb.append("/"+ze.getCompressedSize());
		}
		return (sb.toString());
	}

	/**
	* Is a test driver. Given a jar file and a resource name, it trys to
	* extract the resource and then tells us whether it could or not.
	*
	* <strong>Example</strong>
	* Let's say you have a JAR file which jarred up a bunch of gif image
	* files. Now, by using JarResources, you could extract, create, and display
	* those images on-the-fly.
	* <pre>
	*     ...
	*     JarResources JR=new JarResources("GifBundle.jar");
	*     Image image=Toolkit.createImage(JR.getResource("logo.gif");
	*     Image logo=Toolkit.getDefaultToolkit().createImage(
	*                   JR.getResources("logo.gif")
	*                   );
	*     ...
	* </pre>
	*/
	/*public static void main(String[] args) throws IOException {
	   if (args.length!=2) {
		  System.err.println(
			 "usage: java JarResources <jar file name> <resource name>"
			 );
		  System.exit(1);
	   }
	   JarResources jr=new JarResources(args[0]);
	   byte[] buff=jr.getResource(args[1]);
	   if (buff==null) {
		  System.out.println("Could not find "+args[1]+".");
	   } else {
		  System.out.println("Found "+args[1]+ " (length="+buff.length+").");
	   }
	}*/

}	// End of JarResources class.
