/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// WLGlobalsHandler.cpp
//
//////////////////////////////////////////////////////////////////////

#include "wlmonitorglobalshandler.h"
#include "wlloadgeneratorglobalshandler.h"
#include "globalvariableproperties.h"
#include "associativearray.h"
#include "loadagendamanager.h"
#include "loadengineinifileproperties.h"
#include "tracedll.h"

#include "Log.h"


//////////////////////////////////////////////////////////////////////
// Implementation of CWLMonitorGlobalsHandler class.
//
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWLMonitorGlobalsHandler::CWLMonitorGlobalsHandler(const char *baseName): 
CNetMessageHandler(baseName)
{
	m_SenderName += "GlobalsHandler";
	Init();
}

CWLMonitorGlobalsHandler::~CWLMonitorGlobalsHandler()
{
}


//////////////////////////////////////////////////////////////////////
// Methods
//////////////////////////////////////////////////////////////////////

void CWLMonitorGlobalsHandler::SendMessageErrorHandler(CAssociativeArray& message , const char* sendTo, const char* errorMsg, bool status)
{
		
}

void CWLMonitorGlobalsHandler::CleanGlobalsDictionary(void)
{
	m_GlobalsTable.clearAndDestroy();
}

void CWLMonitorGlobalsHandler::MessageHandler(CAssociativeArray& request)
{
	try {
		const char *varName = request.GetValueOfParameter("$VARNAME");
		const char *globalObjectName = request.GetValueOfParameter("$GLOBALSNAME");
		const char *methodName = request.GetValueOfParameter("$METHOD");

		
		if(strcmp(methodName, "GET") == 0) { 
			CGlobalVariableProperties* retVal = m_GlobalsTable.Get(globalObjectName, varName);	
			SendReplyData(request, retVal);
		}
		else {
			CGlobalVariableProperties *properties = 
				new CGlobalVariableProperties(request.GetValueOfParameter("$TYPE"), 
																			request.GetValueOfParameter("$VALUE"));
			
			if(strcmp(methodName, "SET") == 0) { 		 
				m_GlobalsTable.Set(globalObjectName, varName, properties);
			}
			else if(strcmp(methodName, "ADD") == 0) {
				m_GlobalsTable.Add(globalObjectName, varName, properties);
			}
			else if(strcmp(methodName, "SADD") == 0) {
				CGlobalVariableProperties* retVal = m_GlobalsTable.SyncAdd(globalObjectName, varName, properties);
				SendReplyData(request, retVal);				
			}
		}
	}
	catch(...) {
		request.AddCouple("$WLGLOBALSERROR", "Exception thrown in Meta Globals");
		const char* sendTo = request.GetValueOfParameter("$FROM");
		SendMessage(request, sendTo, "");
	}
}

void CWLMonitorGlobalsHandler::SendReplyData(CAssociativeArray& request, CGlobalVariableProperties* properties)
{
	const char *realAgendaName = request.GetValueOfParameter("$GLOBALSREALNAME");
	
	if(IsAgendaPaused(realAgendaName) || IsAgendaStoped(realAgendaName)) {
				return;
	}
	
	request.AddCouple("$TYPE", properties->GetType());
	request.AddCouple("$VALUE", properties->GetUnformatedCharValue());
	
	// send Get's value back to the requester
	const char* sendTo = request.GetValueOfParameter("$FROM");
	SendMessage(request, sendTo, realAgendaName); // Add Error msg 
}

//////////////////////////////////////////////////////////////////////
// Implementation of CWLMonitorGlobalsHandler class.
//
//////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWLLoadGeneratorGlobalsHandler::CWLLoadGeneratorGlobalsHandler(void)
{
	m_MessagesQueue.Init();

#if !defined(RV_THR_CANT_EXTRACT_TEMPLATE_ARGS_FROM_SIGNATURE)            
	m_MessagesHandlingThread = rvMakeThreadFunction(*this, 
														 &CWLLoadGeneratorGlobalsHandler::MainLoop);   
#else
	m_MessagesHandlingThread = rvMakeThreadFunctionM( 
														 CWLLoadGeneratorGlobalsHandler,
														 *this, void, 
											    	 &CWLLoadGeneratorGlobalsHandler::MainLoop);   
#endif

	m_MessagesHandlingThread.start();
}

CWLLoadGeneratorGlobalsHandler::~CWLLoadGeneratorGlobalsHandler(void)
{
	Terminate();
}


//////////////////////////////////////////////////////////////////////
// Methods
//////////////////////////////////////////////////////////////////////

void CWLLoadGeneratorGlobalsHandler::MainLoop(void)
{
	TIMERENTRY("CWLLoadGeneratorGlobalsHandler::MainLoop");
	Request *newRequest;
	CTraceMsg::RegisterThread ();

	RVThreadSelf self = ::rvThread();
#ifndef __TV_linux__
  self.setPriority(self.getMaxPriority() - 1);
#endif // __TV_linux__
	
	while(newRequest = (Request*)m_MessagesQueue.waitEvent()) {
		CAssociativeArray *request = newRequest->m_Request;

		try {
			const char *varName = request->GetValueOfParameter("$VARNAME");
			const char *globalObjectName = request->GetValueOfParameter("$GLOBALSNAME");
			const char *methodName = request->GetValueOfParameter("$METHOD");
			
		
			if(strcmp(methodName, "GET") == 0) {
				if(m_AgendaManager->IsAgendaPaused()) {
					return;
				}
				CGlobalVariableProperties *properties = m_GlobalsTable.Get(globalObjectName, varName);
			
				request->AddCouple("$TYPE", properties->GetType());
				request->AddCouple("$VALUE", properties->GetUnformatedCharValue());

				// send Get's value back to the requester(LGSlave)
				newRequest->m_SlaveProxy->SendMessageToSlave(request);
			}
			else {
				CGlobalVariableProperties *properties = 
					new CGlobalVariableProperties(request->GetValueOfParameter("$TYPE"), 
																				request->GetValueOfParameter("$VALUE"));

				if(strcmp(methodName, "SET") == 0) { 
					m_GlobalsTable.Set(globalObjectName, varName, properties);
				}
				else if(strcmp(methodName, "ADD") == 0) {	
					m_GlobalsTable.Add(globalObjectName, varName, properties);
				}
				delete request;
			}
		}
		catch(...) {
			CAssociativeArray *error = new CAssociativeArray();
			error->AddCouple("$WLGLOBALSERROR", "Exception thrown in Generator Globals");
			newRequest->m_SlaveProxy->SendMessageToSlave(error);
			break;
		}
		delete newRequest;
	}		
}

void CWLLoadGeneratorGlobalsHandler::SetLoadAgendaManager(CLoadAgendaManager *agendaManager)
{
	m_AgendaManager = agendaManager;
}

void CWLLoadGeneratorGlobalsHandler::MessageHandler(CAssociativeArray& request, CLoadGeneratorSlaveProxy *slaveProxy)
{
	Request *newRequest = new Request();
	CAssociativeArray *requestArray = new CAssociativeArray();

	requestArray->FromString(request.ToString());
	newRequest->m_Request = requestArray;
	newRequest->m_SlaveProxy = slaveProxy;

	m_MessagesQueue.putEvent(newRequest);
}

void CWLLoadGeneratorGlobalsHandler::Terminate(void)
{
	m_MessagesQueue.release();
	m_MessagesHandlingThread.join(CLEIniFileProperties::WaitForThreadtoTerminate);
	m_MessagesQueue.Terminate();
}
