/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// RFS.h: interface for the RFS class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_RFS_H__352CB79D_CE23_49B7_B520_FC4030ED8579__INCLUDED_)
#define AFX_RFS_H__352CB79D_CE23_49B7_B520_FC4030ED8579__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include "filesassociationarchive.h"
#include "rfsmessageslistener.h"
#include "types.h"


class RFSEvent;
class RVCString;
class RFSDataObject;
class CNetMessageHandler;
class RVCriticalSection;


class DllExport RFS : public RFSMessagesListener
{
public:
	enum DirectorySym { Default, InstallationDir, TempDir, UserDir };

	//
	// all created and deleted directories must
	// apply this token, i.e  c:\temp\SECURITY_PATH_TOKEN\mydir
	static const char* SECURITY_PATH_TOKEN;

public:
	//
	// creates RFS object with new defualt connection
	RFS();

	//
	// creates RFS object, gets CNetMessageHandler as connection
	RFS(CNetMessageHandler* connection);
	virtual ~RFS();

	//
	// return: the remote dirctory path in which the files are placed
	// throws RFSException incase and error occurred
	virtual bool SendArchive(const char* remoteHost, FilesAssociationArchive* archive);

	//
	// return: the local dirctory path in which the files are placed
	// throws RFSException incase and error occurred
	virtual bool GetArchive(const char* remoteHost, FilesAssociationArchive* archive);

	//
	// create base working directory path on a remote machine
	// that ends with \\SECURITY_PATH_TOKEN\\myhostname
	// for example C:\Program Files\RadView\TestView\rvtemp\10.0.1.19 
	// were 10.0.1.19 is the ip of the local machine
	// @@ return NULL incase an error occurred 
	virtual RFSString CreateRemoteBaseDirectory(const char* remoteHost, DirectorySym dirSym, bool deleteOldBaseDir);

	//
	// create working directory path on a remote machine
	// the path must contain the SECURITY_PATH_TOKEN as one of its directories
	// @@ return NULL incase an error occurred 
	virtual RFSString CreateRemoteDirectory(const char* remoteHost, const char* dirPath, bool deleteOldDir = false);

	//
	// delete a remote directory path (including all sub directories and files)
	// the directory path must contain the SECURITY_PATH_TOKEN as one of its directories
	// @@ return NULL incase an error occurred 
	virtual RFSString DeleteRemoteDirectory(const char* remoteHost, const char* dirPath);

  //
	// delete a remote file
	// the file path must contain the SECURITY_PATH_TOKEN as one of its directories
	// @@ return NULL incase an error occurred 
	virtual RFSString DeleteRemoteFile(const char* remoteHost, const char* fileName);

	//
	// return a token that must be contained in any remote path 
	// that should be created or deleted. the token must appear
	// as a directory name.
	virtual RFSString GetSecurityPathToken();

	//
	// return TV_TOP value from the remote running TestTalk, , NULL for error
	virtual RFSString GetRemoteInstallationDir(const char* remoteHost);

	//
	// return temp dir from the remote host, NULL for error
	virtual RFSString GetRemoteTempDirectory(const char* remoteHost);

	//
	// return user dir from the remote host, NULL for error
	virtual RFSString GetRemoteUserDir(const char* remoteHost);


	//
	// return an error description incase an error occurred
	virtual RFSString GetLastErrorMessage();

public:
	void CopyFileDoneDispatched(int copyFileCompletionState,
															const char* copyStatusFileName,
															const char* localFileName, 
															const char* remoteFileName, 
															const char* remoteHostAddress,
															CAssociativeArray& message);

	RFSDataObject* CopyFile(const char* remoteHost, const char *srcFile, const char *dstFile);

	virtual void* SendRFSRequest(const char* remoteHost, const char* requestSymbol, CAssociativeArray* sendMessage);
	virtual void  SendRFSResponse(const char* remoteHost, const char* responseSymbol, CAssociativeArray* sendMessage);

	virtual void ProcessRFSRequest(CAssociativeArray& message, RVCString& symbol);
	virtual void ProcessRFSResponse(CAssociativeArray& message, RVCString& symbol);

	bool CheckRemotePathAccessibility(const char* dirPath);

	virtual const char* RemoveRemoteFile(const char* remoteHost, const char* fileName, bool isDirectory);

	bool IsNull(RFSString& str);

protected:
	virtual const char* GetRemoteDirectory(const char* remoteHost, const char* dirSymbol);

protected:
	bool                mIsConnectionOwner;
	CNetMessageHandler* mConnection;
	RFSEvent*           mRFSOperationEvent;
	RVCString*          mUniqueAndSecuredPathToken;
	RVCString*          mGlobalRetVal;
	RVCriticalSection*  mOperationSection;
};



class DllExport RFSException
{
public:
	RFSException();
	RFSException(RFSException& exception);
	RFSException(const char* message);
	virtual ~RFSException();

	inline const char* GetMessage() {return mMessage.data();}

protected:
	RVCString mMessage;
};


#endif // !defined(AFX_RFS_H__352CB79D_CE23_49B7_B520_FC4030ED8579__INCLUDED_)
