/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// UDPReceiver.cpp: implementation of the UDPReceiver class.
//
//////////////////////////////////////////////////////////////////////

#include "udpreceiver.h"
#include "tracedll.h"
#ifdef __TV_UNIX__
#include <sys/time.h>
#endif
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

UDPReceiver::UDPReceiver(void)
{
	m_Running = m_Initialized = false;

// BEGIN: create a functor
#if !defined(RV_THR_CANT_EXTRACT_TEMPLATE_ARGS_FROM_SIGNATURE)
	m_ReceiverThread = rvMakeThreadFunction(
														 *this,
														 &UDPReceiver::ThreadLoop);
#else
	m_ReceiverThread = rvMakeThreadFunctionM(
														 UDPReceiver,
														 *this, void,
											    	 &UDPReceiver::ThreadLoop);
#endif
// END: create a functor
}

UDPReceiver::~UDPReceiver(void)
{
	m_Running = m_Initialized = false;
}

int UDPReceiver::Init(u_short port)
{
#ifdef __TV_WINDOWS__
	WSADATA Data;
#endif
	int status, enable = 1;

	if(m_Initialized)
		return true;

#ifdef __TV_WINDOWS__
  if(WSAStartup(MAKEWORD(1, 1), &Data) != 0) {
    WlTraceMsg("ERROR: UDPReceiver::Init - WSAStartup unsuccessful");
		return false;
	}
#endif

	// zero the sockaddr_in structure
  memset(&m_RecvSockAddr, 0, sizeof(m_RecvSockAddr));

  // specify the port portion of the address
  m_RecvSockAddr.sin_port = htons(port);

  // specify the address family as Internet
  m_RecvSockAddr.sin_family = AF_INET;

  // specify that the address does not matter
  m_RecvSockAddr.sin_addr.s_addr = htonl(INADDR_ANY);

  // create a socket
  m_RecvSocket = socket(AF_INET, SOCK_DGRAM, 0);
  if(m_RecvSocket == INVALID_SOCKET) {
    WlTraceMsg("ERROR: UDPReceiver::Init - socket unsuccessful");
		return false;
	}

  // associate the socket with the address
  status = bind(m_RecvSocket, (struct sockaddr *)&m_RecvSockAddr, sizeof(m_RecvSockAddr));
  if(status == SOCKET_ERROR) {
		WlTraceMsg("ERROR: UDPReceiver::Init - bind unsuccessful");
		return false;
	}

	m_Initialized = true;

	return true;
}

int UDPReceiver::Start(void)
{
	int startStatus;

	// start the thread and wait for it's execution:
	m_Running = true;
	startStatus = m_ReceiverThread.start();
	m_ReceiverThread.wait(RV_THR_RUNNING);

	return startStatus;
}

void UDPReceiver::ThreadLoop(void)
{
	// BEGIN: Set priority
	{
		RVThreadSelf self = ::rvThread();
		self.setPriority(self.getMaxPriority());
	}
	// END: Set priority

  char buffer[MAXBUFLEN];

	// start the receiving loop.
	// the loop will receive data from any client that
	// sends it's data to this port.
	while(m_Running) {
		if( Run(buffer, MAXBUFLEN) ) {
			MessageHandler(buffer);
		}
		else {
			//Terminate();
		}
  }
}

int UDPReceiver::Run(/*OUT*/char* buffer, /*IN*/int bufferLen)
{
	if( !ReceiveFrom(buffer, bufferLen) ) {
		if(m_Running) {
			WlTraceMsg("ERROR: UDPReceiver::Run - recvfrom unsuccessful");
			return false;
		}
		return false;
	}
	return true;
}

void UDPReceiver::Stop(void)
{
	m_Running = false;

	// terminate net conection
	Terminate();

	// wait for thread to exit for 5 sec' and terminate
	// it if it didn't succeed to exit by it self.
	if(m_ReceiverThread.join(5000) == RV_THR_TIMEOUT) {
		m_ReceiverThread.terminate();
	}
}

int UDPReceiver::ReceiveFrom(char* buffer, size_t bufferLen)
{
	int numrcv;

  numrcv = recvfrom(m_RecvSocket, buffer, bufferLen,
										NO_FLAGS_SET, NULL, NULL);

  if(numrcv == SOCKET_ERROR) {
		return false;
  }
	return true;
}

int UDPReceiver::SocketReadyForRead(long seconds, long microseconds)
{
	timeval        wait;
	fd_set  readfds;

#ifdef __TV_WINDOWS__
	readfds.fd_count = 1;
	readfds.fd_array[0] = m_RecvSocket;
#else
	FD_ZERO(&readfds);
	FD_SET(m_RecvSocket, &readfds);
#endif
	wait.tv_sec = seconds;
	wait.tv_usec = microseconds;

	// returns an integer > 0 if there is ready data to read
	// from the socket and 0 else.
	return select(0, &readfds, NULL, NULL, &wait);
}

void UDPReceiver::Terminate(void)
{
	if(!m_Initialized)
		return;

	m_Initialized = false;

#ifdef __TV_WINDOWS__
	int status = closesocket(m_RecvSocket);
#else
	int status = close(m_RecvSocket);
#endif
  if(status == SOCKET_ERROR)
		WlTraceMsg("ERROR: UDPReceiver::Terminate - closesocket unsuccessful");

#ifdef __TV_WINDOWS__
  status = WSACleanup();
  if(status == SOCKET_ERROR)
    WlTraceMsg("ERROR: UDPReceiver::Terminate - WSACleanup unsuccessful");
#endif
}

bool UDPReceiver::ReceiverStatusRunning(void)
{
	return m_Running;
}

void UDPReceiver::MessageHandler(const char* buffer)
{
	cout<<buffer<<endl;
	WlTraceMsg("UDPReceiver::MessageHandler - received: %s", buffer);
}
